/*
  WizTemp -- common functions
  Copyright (C) 1990,1991,1992,1993 by Network Wizards

  init_port(char *tty)                initialize tty port
  float get_temp()                    get a sample from wiztemp in Kelvin
  close_port()                        close down tty port
  dtr(int state)                      play with dtr
  time_t mkltime(struct tm *timbrk)   convert a struct tm to a time_t
  fatal(char *message)                display a message and then die()
  die()                               print last error and then halt
*/

#define DEBUG FALSE

#include <stdio.h>
#include <sys/types.h>
#include <time.h>
#include <sys/time.h>
#include <fcntl.h>
#include "wiztemp.h"
#if HAVE_NOTIME
#include <sys/timeb.h>
#endif
#if HAVE_SGTTY
#include <sgtty.h>
#endif
#if HAVE_TERMIOS
#include <termios.h>
#endif
#if PC
#include <dos.h>
#endif

#ifndef TCGETS
#define TCGETS TCGETA
#define TCSETS TCSETA
#endif

extern int errno;

static int ftty;       /* tty file descriptor of wiztemp */
static int pc_port;
static int port;

#if HAVE_SELECT
static fd_set ttyfdset;
static struct timeval ttytout;
#endif


/*
 Initialize or open comm port for wiztemp.
 Arg is tty or port designation string.
*/
init_port(tty)
  char *tty;
{
#if UNIX
#if HAVE_SGTTY
  struct sgttyb ttyargs;
#endif
#if HAVE_TERMIOS
  struct termios ttyargs;
#endif

#if DEBUG
  printf("init_port(%s)\n",tty);
#endif
  if ((ftty = open(tty,O_RDWR)) == -1)
  {
    fprintf(stderr,"?Can't open tty '%s'.\n",tty);
    die();
  }

#if HAVE_SGTTY
  ttyargs.sg_ispeed = B9600;
  ttyargs.sg_ospeed = B9600;
  ttyargs.sg_erase = 0;
  ttyargs.sg_kill = 0;
  ttyargs.sg_flags = RAW;
  if ((ioctl(ftty, TIOCSETP, (char *) &ttyargs)) == -1)
  {
    fprintf(stderr,"?Can't do ioctl TIOCSETP on tty '%s'.\n",tty);
    die();
  }
#endif

#if HAVE_TERMIOS
  if ((ioctl(ftty, TCGETS, (char *) &ttyargs)) == -1)
  {
    fprintf(stderr,"?Can't do ioctl TCGETS on tty '%s'.\n",tty);
    die();
  }
  ttyargs.c_iflag = 0;
  ttyargs.c_oflag = 0;
  ttyargs.c_cflag = (B9600 | CS8 | CREAD | CLOCAL);
  ttyargs.c_lflag = 0;
  if ((ioctl(ftty, TCSETS, (char *) &ttyargs)) == -1)
  {
    fprintf(stderr,"?Can't do ioctl TSETS on tty '%s'.\n",tty);
    die();
  }
#endif

#if HAVE_SELECT
  /* setup params for select call for function get_temp */
  ttytout.tv_sec =  1;
  ttytout.tv_usec = 0;
  FD_ZERO(&ttyfdset);
#endif
#endif


#if PC
  int far *port_addr;

#if DEBUG
  printf("init_port(%s)\n",tty);
#endif
  pc_port = com_port(tty);
  port_addr = MK_FP(0x0040, pc_port*2);
  if (*port_addr == NULL)
  {
    fprintf(stderr,"?Port '%s' doesn't exist.\n",tty);
    die();
  }
  port = *port_addr;
  bioscom(0,(0xE0|0x03),pc_port); /* 9600, 8 bits, no parity, 1 stop */
#endif

#if (MODEL==1)
  dtr(OFF);
#endif
#if (MODEL==2)
  dtr(ON);
#endif
}


/* get a reading from the wiztemp, return in DegK, or -1 if problem */
float get_temp()
{
  unsigned char buf[8];
  int i, cnt, status, len;
  float dk;

#if (MODEL==1)
  dtr(ON);
#endif

#if UNIX
  dk = -1;			    /* error if no temp reading */
  for (cnt = 0; cnt < 3; cnt++)	    /* attempt to get temp 3 times */
  {
#if HAVE_SELECT
    /* flush input buffer */
    FD_SET(ftty,&ttyfdset);
    if (select(FD_SETSIZE,&ttyfdset,NULL,NULL,&ttytout) > 0)
      read(ftty,buf,1);
#endif

    /* send command to get temperature */
#if (MODEL==1)
    *buf = 'U';
#endif
#if (MODEL==2)
    *buf = '\r';
#endif
    if (write(ftty,buf,1) != 1)
      fatal("Problem sending command to WizTemp");

#if (MODEL==2)
#if HAVE_SELECT
    FD_SET(ftty,&ttyfdset);
    /* read response, if any */
    for (i = 0; i < 8; i++)
    {
      status = select(FD_SETSIZE,&ttyfdset,NULL,NULL,&ttytout);
      if (status < 1) break;
      read(ftty,buf+i,1);
      buf[i] = buf[i] & 0x7f;
      if (buf[i] == '\r') break;
    }
    if ((i == 8) || (status < 1)) continue;
#else
    for (i = 0; i < 8; i++)
    {
      read(ftty,buf+i,1);
      buf[i] = buf[i] & 0x7f;
      if (buf[i] == '\r') break;
    }
    if (i == 8) continue;
#endif
    len = i;

#if DEBUG
    buf[len] = '\0';
    printf("got %s\n",buf);
#endif

    /* validate response */
    status = 0;
    len--;
    for (i = 0; i < len; i++)
      status = status ^ buf[i];
    if ((status + 63) != buf[len]) continue;

    /* overrange (hard error)? */
    if ((buf[0] == '+') || (buf[0] == '-')) break;

    buf[len] = '\0';
    if (sscanf(buf,"%f",&dk) != 1)
      dk = -1;
    break;
#endif /* WizTemp-2 */

#if (MODEL==1)
#if HAVE_SELECT
    FD_SET(ftty,&ttyfdset);
    status = select(FD_SETSIZE,&ttyfdset,NULL,NULL,&ttytout);
    if (status < 1) continue;
#endif
    read(ftty,buf,1);
#if DEBUG
    printf("got %d\n",(int) buf[0]);
#endif
    /* overrange (hard error)? */
    if ((buf[0] == 0) || (buf[0] == 255)) break;
    /* ok, convert to Kelvin */
    dk = (buf[0] * TRES) + TBASE;
    break;
#endif /* WizTemp-1 */

  }
#endif /* UNIX */

#if PC
  cnt = 0;
  while (!(inportb(port+5) & 0x20) && cnt < 10000) cnt++;
  if (cnt>=10000)
    fatal("Problem writing data to WizTemp");
  outportb(port,'U');
  sleep(1);
  if (!(inportb(port+5) & 0x01))
    c = 0;
  else 
    c = (unsigned char) inportb(port);

  /* don't use bios cause it checks modem signals
  bioscom(1,'U',pc_port);
  if ((c = (unsigned char) bioscom(2,0,pc_port)) > 255)
    fatal("Problem reading data from WizTemp");
*/
#endif

#if (MODEL==1)
  dtr(OFF);
#endif

  return(dk);
}

/* close tty port */
close_port()
{
#if UNIX
  close(ftty);
#endif
}


/* Turn DTR (and RTS if possible and WizTemp-2) ON or OFF. */
#if UNIX
dtr(state)
  int state;
{
  if (state)
  {
#if HAVE_SDTR
    if ((ioctl(ftty, TIOCSDTR, 0)) == -1)
      fatal("Can't set DTR -- ioctl TIOCSDTR failed.");
#endif

#if HAVE_MBIS
    int bits;

    bits = (TIOCM_DTR | TIOCM_RTS);
    if ((ioctl(ftty, TIOCMBIS, &bits)) == -1)
      fatal("Can't set DTR and RTS -- ioctl TIOCMBIS failed.");
#endif
  }
  else
  {
#if HAVE_SDTR
    if ((ioctl(ftty, TIOCCDTR, 0)) == -1)
      fatal("Can't clear DTR -- ioctl TIOCCDTR failed.");
#endif

#if HAVE_MBIS
    int bits;

    bits = (TIOCM_DTR | TIOCM_RTS);
    if ((ioctl(ftty, TIOCMBIC, &bits)) == -1)
      fatal("Can't clear DTR and RTS -- ioctl TIOCMBIS failed.");
#endif
  }
}
#endif

#if PC
dtr(state)
  int state;
{
  if (state)
  {
    outportb(port+4,1);
    sleep(1);
  }
  else
  {
    outportb(port+4,0);
    sleep(1);
  }
}

int com_port(port)
  char *port;
{
  if (((*port == 'C') || (*port == 'c')) &&
      ((*(port+1) == 'O') || (*(port+1) == 'o')) &&
      ((*(port+2) == 'M') || (*(port+2) == 'm')))
    return(*(port+3)-'1');
  else
    fatal("Ports are specified as 'COMn'.");    
}
#endif


fatal(line)
  char *line;
{
  fprintf(stderr,"?%s\n",line);
  die();
}

die()
{
  if (errno)
    perror("Last error");
  exit(1);
}



#if HAVE_MKTIME
time_t mkltime(timbrk)
  struct tm *timbrk;
{
  return(mktime(timbrk));
}
#endif

#if HAVE_TIMELOCAL
time_t mkltime(timbrk)
  struct tm *timbrk;
{
  return(timelocal(timbrk));
}
#endif

#if HAVE_NOTIME
/*
  mkltime() -- takes a struct tm and returns a time_t.
  Under SunOS this function is called timelocal().
  Under ANSI C this function is called mktime().
*/

#define isleap(y) (((y) % 4) == 0 && ((y) % 100) != 0 || ((y) % 400) == 0)

static int dmsize[12] =  { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

time_t mkltime(timbrk)
  struct tm *timbrk;
{
  int i, year, month, day, hour, mins, secs;
  time_t tv;
  struct timeb timb;

  year = timbrk->tm_year;
  month = timbrk->tm_mon;
  day = timbrk->tm_mday;
  hour = timbrk->tm_hour;
  mins = timbrk->tm_min;
  secs = timbrk->tm_sec;

  tv = 0;
  for (i = 70; i < year; i++)
    tv += daysize(1900 + i);
  /* if leap year */
  if (daysize(1900 + year) == 366 && month >= 3)
    tv++;
  while (month)
    tv += dmsize[--month];
  tv += day-1;
  tv = 24*tv + hour;
  tv = 60*tv + mins;
  tv = 60*tv + secs;
  ftime(&timb);
  /* account for our local timezone */
  tv += timb.timezone * 60;
  /* now fix up any local daylight time */
  if (timb.dstflag)
    tv -= 60 * 60;
  return (tv);
}

daysize(y)
  int y;
{
  if (isleap(y))
    return(366);
  return(365);
}
#endif

/* names of months, in English */
char *montab[] = {
  "???",
  "Jan",
  "Feb",
  "Mar",
  "Apr",
  "May",
  "Jun",
  "Jul",
  "Aug",
  "Sep",
  "Oct",
  "Nov",
  "Dec"
};
